@@ -2,37 +2,41 @@
 #'
 #' Fast similarity/distance computation function for large sparse matrices. You
 #' can floor small similarity value to to save computation time and storage
-#' space by an arbitrary threshold (\code{min_simil}) or rank (\code{rank}). You
+#' space by an arbitrary threshold (`min_simil`) or rank (`rank`). You
 #' can specify the number of threads for parallel computing via
 #' `options(proxyC.threads)`.
 #'
-#' @param x \link{matrix} or \link{Matrix} object. Dense matrices are covered to
-#'   the \link{CsparseMatrix-class} internally.
-#' @param y if a \link{matrix} or \link{Matrix} object is provided, proximity
-#'   between documents or features in \code{x} and \code{y} is computed.
+#' @param x a [base::matrix] or [Matrix::Matrix] object. Dense matrices are covered to
+#'   the [Matrix::CsparseMatrix-class] internally.
+#' @param y if a [base::matrix] or [Matrix::Matrix] object is provided, proximity
+#'   between documents or features in `x` and `y` is computed.
 #' @param margin integer indicating margin of similarity/distance computation. 1
 #'   indicates rows or 2 indicates columns.
 #' @param method method to compute similarity or distance
+#' @param mask a pattern matrix created using [mask()] for masked similarity/distance computation.
+#'  The shape of the matrix must be the same as the resulting matrix.
 #' @param min_simil the minimum similarity value to be recorded.
 #' @param rank an integer value specifying top-n most similarity values to be
 #'   recorded.
-#' @param p weight for Minkowski distance
-#' @param drop0 if \code{TRUE}, zero values are removed regardless of
-#'   \code{min_simil} or \code{rank}.
-#' @param diag if \code{TRUE}, only compute diagonal elements of the
+#' @param p weight for Minkowski distance.
+#' @param drop0 if `TRUE`, removes zero values to make the
+#'   similarity/distance matrix sparse. It has no effect when `dense = TRUE`.
+#' @param diag if `TRUE`, only compute diagonal elements of the
 #'   similarity/distance matrix; useful when comparing corresponding rows or
 #'   columns of `x` and `y`.
-#' @param use_nan if `TRUE`, return `NaN` if the standard deviation of a vector
+#' @param use_nan if `TRUE`, returns `NaN` if the standard deviation of a vector
 #'   is zero when `method` is "correlation"; if all the values are zero in a
 #'   vector when `method` is "cosine", "chisquared", "kullback", "jeffreys" or
 #'   "jensen". Note that use of `NaN` makes the similarity/distance matrix
 #'   denser and therefore larger in RAM. If `FALSE`, return zero in same use
 #'   situations as above. If `NULL`, will also return zero but also generate a
 #'   warning (default).
+#' @param sparse if `TRUE`, returns [Matrix::sparseMatrix] object. When neither
+#'   `min_simil` nor `rank` is used, dense matrices require less space in RAM.
 #' @param digits determines rounding of small values towards zero. Use primarily
-#'   to correct rounding errors in C++. See \link{zapsmall}.
-#' @details
-#'   ## Available Methods
+#'   to correct floating point errors. Rounding is performed in C++ in a similar
+#'   way as [base::zapsmall].
+#' @details ## Available Methods
 #'
 #'   Similarity:
 #' \itemize{
@@ -80,15 +84,17 @@
 #' mt <- Matrix::rsparsematrix(100, 100, 0.01)
 #' simil(mt, method = "cosine")[1:5, 1:5]
 simil <- function(x, y = NULL, margin = 1,
-                  method = c("cosine", "correlation", "jaccard", "ejaccard", "fjaccard",
-                             "dice", "edice", "hamann", "faith", "simple matching"),
+                  method = c("cosine", "correlation", "dice", "edice",
+                             "jaccard", "ejaccard", "fjaccard",
+                             "hamann", "faith", "simple matching"),
+                  mask = NULL,
                   min_simil = NULL, rank = NULL, drop0 = FALSE, diag = FALSE,
-                  use_nan = NULL, digits = 14) {
+                  use_nan = NULL, sparse = TRUE, digits = 14) {
 
     method[method == "hamman"] <- "hamann" # for transition
     method <- match.arg(method)
-    proxy(x, y, margin, method, min_proxy = min_simil, rank = rank, drop0 = drop0,
-          diag = diag, use_nan = use_nan, digits = digits)
+    proxy(x, y, margin, method, mask = mask, min_proxy = min_simil, rank = rank, drop0 = drop0,
+          diag = diag, use_nan = use_nan, sparse = sparse, digits = digits)
 
 }
 
@@ -103,22 +109,25 @@ simil <- function(x, y = NULL, margin = 1,
 dist <- function(x, y = NULL, margin = 1,
                  method = c("euclidean", "chisquared", "kullback", "jeffreys", "jensen",
                             "manhattan", "maximum", "canberra", "minkowski", "hamming"),
-                 p = 2, smooth = 0, drop0 = FALSE, diag = FALSE, use_nan = NULL, digits = 14) {
+                 mask = NULL,
+                 p = 2, smooth = 0, drop0 = FALSE, diag = FALSE, use_nan = NULL,
+                 sparse = TRUE, digits = 14) {
 
     method <- match.arg(method)
-    proxy(x, y, margin, method, p = p, smooth = smooth, drop0 = drop0,
-          diag = diag, use_nan = use_nan, digits = digits)
+    proxy(x, y, margin, method, mask = mask, p = p, smooth = smooth, drop0 = drop0,
+          diag = diag, use_nan = use_nan, sparse = sparse, digits = digits)
 }
 
 #' @import Rcpp
 #' @useDynLib proxyC
 proxy <- function(x, y = NULL, margin = 1,
-                  method = c("cosine", "correlation", "jaccard", "ejaccard", "fjaccard",
+                  method = c("cosine", "correlation", "product", "jaccard", "ejaccard", "fjaccard",
                              "dice", "edice", "hamann", "simple matching", "faith",
                              "euclidean", "chisquared", "kullback", "jeffreys", "jensen",
                              "manhattan", "maximum", "canberra", "minkowski", "hamming"),
+                  mask = NULL,
                   p = 2, smooth = 0, min_proxy = NULL, rank = NULL, drop0 = FALSE,
-                  diag = FALSE, use_nan = NULL, digits = 14) {
+                  diag = FALSE, use_nan = NULL, sparse = TRUE, digits = 14) {
 
     method[method == "hamman"] <- "hamann" # for transition
     method <- match.arg(method)
@@ -143,6 +152,15 @@ proxy <- function(x, y = NULL, margin = 1,
         if (nrow(x) != nrow(y))
             stop("x and y must have the same number of rows")
     }
+    if (is.null(mask)) {
+        mask <- as(Matrix(numeric(), nrow = 0, ncol = 0, sparse = TRUE), "CsparseMatrix")
+        use_mask <- FALSE
+    } else {
+        mask <- as(as(as(mask, "CsparseMatrix"), "generalMatrix"), "dMatrix")
+        use_mask <- TRUE
+        if (nrow(mask) != ncol(x) || ncol(mask) != ncol(y))
+            stop(sprintf("The shape of mask must be %d x %d.", ncol(x), ncol(y)))
+    }
     if (is.null(min_proxy))
         min_proxy <- -1.0
     if (is.null(rank))
@@ -174,9 +192,6 @@ proxy <- function(x, y = NULL, margin = 1,
         weight <- 2
     } else if (method == "dice") {
         boolean <- TRUE
-        method <- "edice"
-    } else if (method == "edice") {
-        weight <- 2
     } else if (method == "hamann") {
         boolean <- TRUE
     } else if (method == "faith") {
@@ -192,27 +207,33 @@ proxy <- function(x, y = NULL, margin = 1,
         x <- as(as(x, "lMatrix"), "dMatrix")
         y <- as(as(y, "lMatrix"), "dMatrix")
     }
-    if (method %in% c("cosine", "correlation", "euclidean") && !diag) {
+    if (method %in% c("cosine", "correlation", "euclidean", "product", "dice", "edice") && !diag) {
         result <- cpp_linear(
             mt1 = x,
             mt2 = y,
-            method = match(method, c("cosine", "correlation", "euclidean")),
+            method = match(method, c("product", "cosine", "correlation", "euclidean", "dice", "edice")),
+            mask = mask,
             rank = rank,
             limit = min_proxy,
             symm = symm,
             drop0 = drop0,
             use_nan = use_nan,
+            use_mask = use_mask,
+            sparse = sparse,
+            digits = digits,
             thread = getThreads()
         )
     } else {
         result <- cpp_pair(
             mt1 = x,
             mt2 = y,
-            method = match(method, c("cosine", "correlation", "ejaccard", "edice",
-                                     "hamann", "simple matching", "faith",
-                                     "euclidean", "chisquared", "kullback", "manhattan",
+            method = match(method, c("cosine", "correlation", "euclidean",
+                                     "dice", "edice", "hamann", "simple matching",
+                                     "faith", "ejaccard", "fjaccard",
+                                     "chisquared", "kullback", "manhattan",
                                      "maximum", "canberra", "minkowski", "hamming",
-                                     "jeffreys", "jensen", "fjaccard")),
+                                     "jeffreys", "jensen")),
+            mask = mask,
             rank = rank,
             limit = min_proxy,
             weight = weight,
@@ -221,80 +242,14 @@ proxy <- function(x, y = NULL, margin = 1,
             diag = diag,
             drop0 = drop0,
             use_nan = use_nan,
+            use_mask = use_mask,
+            sparse = sparse,
+            digits = digits,
             thread = getThreads()
         )
     }
     if (diag)
         result <- as(as(result, "diagonalMatrix"), "ddiMatrix")
-    result@x <- zapsmall(result@x, digits)
     dimnames(result) <- list(colnames(x), colnames(y))
     return(result)
 }
-
-#' Standard deviation of columns and rows of large matrices
-#'
-#' Produces the same result as \code{apply(x, 1, sd)} or \code{apply(x, 2, sd)}
-#' without coercing matrix to dense matrix. Values are not identical to
-#' \code{sd} because of the floating point precision issue in C++.
-#' @param x \link{matrix} or \link{Matrix} object
-#' @examples
-#' mt <- Matrix::rsparsematrix(100, 100, 0.01)
-#' colSds(mt)
-#' apply(mt, 2, sd) # the same
-#' @export
-colSds <- function(x) {
-    x <- as(as(x, "CsparseMatrix"), "dMatrix")
-    result <- cpp_sd(x)
-    names(result) <- colnames(x)
-    return(result)
-}
-
-#' @rdname colSds
-#' @export
-rowSds <- function(x) {
-    x <- as(as(x, "CsparseMatrix"), "dMatrix")
-    result <- cpp_sd(t(x))
-    names(result) <- rownames(x)
-    return(result)
-}
-
-#' Count number of zeros in columns and rows of large matrices
-#'
-#' Produces the same result as applying \code{sum(x == 0)} to each row or column.
-#' @param x \link{matrix} or \link{Matrix} object
-#' @examples
-#' mt <- Matrix::rsparsematrix(100, 100, 0.01)
-#' colZeros(mt)
-#' apply(mt, 2, function(x) sum(x == 0)) # the same
-#' @export
-colZeros <- function(x) {
-    x <- as(as(x, "CsparseMatrix"), "dMatrix")
-    result <- nrow(x) - cpp_nz(x)
-    names(result) <- colnames(x)
-    return(result)
-}
-
-#' @rdname colZeros
-#' @export
-rowZeros <- function(x) {
-    x <- as(as(x, "CsparseMatrix"), "dMatrix")
-    result <- ncol(x) - cpp_nz(t(x))
-    names(result) <- rownames(x)
-    return(result)
-}
-
-getThreads <- function() {
-
-    # respect other settings
-    default <- c("tbb" = as.integer(Sys.getenv("RCPP_PARALLEL_NUM_THREADS")),
-                 "omp" = as.integer(Sys.getenv("OMP_THREAD_LIMIT")),
-                 "max" = cpp_get_max_thread())
-    default <- unname(min(default, na.rm = TRUE))
-
-    value <- as.integer(getOption("proxyC.threads", default))
-    if (length(value) != 1 || is.na(value)) {
-        stop("proxyC.threads must be an integer")
-    }
-    return(value)
-}
-
